#include "gstore_shmem.h"
#include "utils/resowner.h"

GraphIndexDSAProcStateStruct *GraphIndexDSAProcState = NULL;

static int GraphIndex_tranche_id(void)
{
	static int tranche_id = 0;

	if (tranche_id == 0)
		tranche_id = LWLockNewTrancheId();

	return tranche_id;
}

GraphIndexDSAProcStateStruct* get_dsa_proc_state(void){
	char *mem;
	dsa_area *area = NULL;
	dsa_pointer data_ptr;

	if(!GraphIndexDSAProcState){
		elog(LOG, "No DSAProc State... hence getting one...");

		LWLockAcquire(GraphIndexShmem->dsa_lock, LW_EXCLUSIVE);

		GraphIndexDSAProcState = (GraphIndexDSAProcStateStruct *)malloc(sizeof(GraphIndexDSAProcStateStruct));
		if(!GraphIndexShmem->graphindex_dsa_handle){
			/* Creating DSA */
			elog(LOG, "Creating DSA...");
			area = dsa_create(GraphIndex_tranche_id());
			/* make sure it doesn't go away even if we do */
			dsa_pin(area);
			dsa_pin_mapping(area);
			/* Saving dsa_handle in shmem for serving other processes */
			GraphIndexShmem->graphindex_dsa_handle = dsa_get_handle(area);
			/* Setting the DSA state for this process */
			GraphIndexDSAProcState->area = area;
			GraphIndexDSAProcState->is_attached = true;
			/* Data for Testing */
			data_ptr = dsa_allocate(area, 42);
			mem = (char *) dsa_get_address(area, data_ptr);
			if (mem != NULL)
			{
				snprintf(mem, 42, "Hello world");
			}
			GraphIndexShmem->graphindex_dsa_ptr = data_ptr;
		}else{
			elog(LOG, "Attaching DSA.....");
			area = dsa_attach(GraphIndexShmem->graphindex_dsa_handle);
			elog(LOG, "Attached area addr ==> %p", area);
			dsa_pin_mapping(area);
			GraphIndexDSAProcState->area = area;
			GraphIndexDSAProcState->is_attached = true;

			mem = (char *) dsa_get_address(area, GraphIndexShmem->graphindex_dsa_ptr);
			if (mem != NULL)
			{
				elog(LOG, "DATA Read after attaching ==> %s", mem);
			}
		}
		
		LWLockRelease(GraphIndexShmem->dsa_lock);
	}
	return GraphIndexDSAProcState;
}