#ifndef GSTORE_SHMEM_H
#define GSTORE_SHMEM_H

#include "postgres.h"
#include "c.h"
#include "pg_config.h"
#include "utils/dsa.h"
#include "utils/hsearch.h"
#include "storage/lwlock.h"

/* Struct that holds Current Process DSA Specific data */
typedef struct {
     dsa_area *area; /* DSA mapped for this process */
     bool is_attached; /* Tells whether this process is attached to this dsa or not */
}GraphIndexDSAProcStateStruct;

/* GraphIndex Shared Memory that holds dsa area handle and other Shmem related contents... */
typedef struct
{
	/* 
	* Used for other processes to attach to the DSA 
	*/
	dsa_handle graphindex_dsa_handle;
	/* 
	* Relative pointer to test data 
	*/
	dsa_pointer graphindex_dsa_ptr;
	/*
	 * Lock protecting from concurrent dsa_area create requests.
	 * Should acquire EXCLUSIVE_LOCK for setting/updating graphindex_dsa_handle member.
	 */
	LWLock *dsa_lock;
	/*
	 * Lock protecting concurrent access to Shared Memory Data(Shmem/DSA as per requirement)
	 * Particularly.. 
	 * 				Should acquire EXCLUSIVE_LOCK for setting/updating sensitive data
	 *				Should acquire SHARED_LOCK for accessing sensitive data
	 */
	LWLock *data_lock;
} GraphIndexShmemStruct;


typedef struct XidDMLDataList {
	char xid;
	dsa_pointer name;
}XidDMLDataList;

extern GraphIndexDSAProcStateStruct *GraphIndexDSAProcState;
extern GraphIndexShmemStruct *GraphIndexShmem;
extern dsa_area *proc_dsa_area;

GraphIndexDSAProcStateStruct* get_dsa_proc_state(void);

#endif /* GSTORE_SHMEM_H */