/*-------------------------------------------------------------------------
 *
 * main.c
 *		Graph Storage Engine for Postgres...
 *
 * Copyright (c) 1996-2016, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
 *		  main/main.c
 *
 *-------------------------------------------------------------------------
 */

#include "postgres.h"
#include "postgres_ext.h"
#include "gstore_shmem.h"
#include "fmgr.h"
#include "utils/builtins.h"
#include "utils/dsa.h"
#include "utils/rel.h"
#include "utils/fmgroids.h"
#include "utils/json.h"
#include "access/amapi.h"
#include "access/genam.h"
#include "access/relscan.h"
#include "access/heapam.h"
#include "access/htup_details.h"
#include "access/itup.h"
#include "access/stratnum.h"
#include "catalog/index.h"
#include "storage/indexfsm.h"
#include "storage/ipc.h"
#include "storage/itemptr.h"
#include "storage/lwlock.h"
#include "storage/shmem.h"
#include "main.h"


PG_MODULE_MAGIC;

PG_FUNCTION_INFO_V1(simple_udf_func);

void _PG_init(void);

void GraphIndexShmemHook(void);

static shmem_startup_hook_type PreviousShmemHook = NULL;
GraphIndexShmemStruct *GraphIndexShmem = NULL;

void _PG_init(void)
{
	RequestAddinShmemSpace(1000000);
	RequestNamedLWLockTranche("GraphIndex_DSA", 1);
	RequestNamedLWLockTranche("GraphIndex_Data", 1);
	PreviousShmemHook = shmem_startup_hook;
  	shmem_startup_hook = GraphIndexShmemHook;
	receiver_initialization();
}

/* 
* Allocations in shared memory
* Creating Shared Memory space to store dsa_handle, so that other backends can access and attach to dsa if needed 
*/
void GraphIndexShmemHook(){
	bool found;

	LWLockAcquire(AddinShmemInitLock, LW_EXCLUSIVE);

	GraphIndexShmem = (GraphIndexShmemStruct *)ShmemInitStruct("GraphIndex_DSA_META",
			sizeof(GraphIndexShmemStruct),
			&found);

	if(!found){
		memset(GraphIndexShmem, 0, sizeof(GraphIndexShmemStruct));
		GraphIndexShmem->dsa_lock = &(GetNamedLWLockTranche("GraphIndex_DSA"))->lock;
		GraphIndexShmem->data_lock = &(GetNamedLWLockTranche("GraphIndex_Data"))->lock;
	}

	LWLockRelease(AddinShmemInitLock);
}

/* SIMPLE UDF FUNCTION */
Datum simple_udf_func(PG_FUNCTION_ARGS)
{
	GraphIndexDSAProcStateStruct *dsa_proc_local_state = get_dsa_proc_state();
	proc_dsa_area = dsa_proc_local_state->area;
	elog(LOG, "proc_dsa_area addr ==> %p", proc_dsa_area);
	dsa_dump(proc_dsa_area);

	/* sample allocations using DSA */

	dsa_pointer XidDMLData;
	XidDMLDataList *XidDMLData_local;
	char *name;
	char *val = "world";
	XidDMLData = dsa_allocate(proc_dsa_area, sizeof(XidDMLDataList));
	XidDMLData_local = (XidDMLDataList *)dsa_get_address(proc_dsa_area, XidDMLData);
	XidDMLData_local->xid = 'h';
	XidDMLData_local->name = dsa_allocate(proc_dsa_area, sizeof(char)*10);
	name = (char *)dsa_get_address(proc_dsa_area, XidDMLData_local->name);
	memcpy(name, val, strlen(val) + 1);

	dsa_dump(proc_dsa_area);
	PG_RETURN_TEXT_P(cstring_to_text("return"));
}
